@@ -32,28 +32,34 @@ def parse_options(argv):
                         help='Provide commit ID.')
     parser.add_argument("--task", "-t", nargs=1, default=[None], metavar='TASK_NAME',
                         help='Provide task_name.')
-    parser.add_argument("--dry", "-D", default=False, action='store_true', help='Perform dry run.')
+    parser.add_argument("--conf-validation", default=False, action='store_true', help='Execute '
+                        'configuration files validation and exit program.')
     parser.add_argument("--keep-locked", "-k", default=False, action='store_true', help='Keep'
                         ' infrastructure locked after task execution.')
     parser.add_argument("--debug", "-d", default=False, action="store_true",
-                        help='Print debug output.')
+                        help='Print debug output. This option does not affect runner output (use'
+                             ' --raw-runner-output for that).')
     parser.add_argument("--syslog", default=False, action="store_true", help='Log warnings and'
                         ' errors to syslog. --debug doesn\'t affect this option!')
     parser.add_argument("--limit", "-l", nargs=1, default=[None], metavar="[LIMIT]",
                         help='Limit task execution to specified host.')
-    parser.add_argument("--conf-dir", "-C", nargs=1, default=[None], metavar="conf_dir",
-                        help='Use non-default configuration directory, only allowed for \
-                              non-binarized exec')
+    parser.add_argument("--conf-dir", nargs=1, default=[None], metavar="conf_dir", help='Use '
+                        'non-default configuration directory, only allowed for non-binarized exec')
     parser.add_argument("--version", "-v", default=False, action="store_true", help='Display'
                             'app version and exit.')
     parser.add_argument("--raw-runner-output", default=False, action="store_true", help='Print'
-                        ' original messages during runner execution instead of formatted ones.')
+                        ' original messages in real time during runner execution.')
     parser.add_argument("--self-setup", nargs=1, default=[None], metavar="LOCAL_SETUP_PATH",
                         help='Setup repo outside of workdir in requested path. This option applies'
                         ' only to infrastructures with allow_user_checkout enabled in infra'
                         ' config!')
     parser.add_argument("--no-color", default=False, action="store_true", help='Disable coloring'
                         'of console messages.')
+    parser.add_argument("--check-mode", "-C", action="store_true", help='Enable --check-mode/-C'
+                        ' when using default runner (ansible-playbook).')
+    parser.add_argument("--dry-mode", "-D", action="store_true", help='Execute default runner'
+                        ' (ansible-playbook) with "ansible_deployer_dry_mode" tag, triggering only'
+                        ' required variable validation in pre_tasks. This tag is not predefined!')
 
     arguments = parser.parse_args(argv)
 
@@ -84,7 +90,7 @@ def parse_options(argv):
     options["stage"] = arguments.stage[0]
     options["commit"] = arguments.commit[0]
     options["task"] = arguments.task[0]
-    options["dry"] = arguments.dry
+    options["conf_val"] = arguments.conf_validation
     options["keep_locked"] = arguments.keep_locked
     options["debug"] = arguments.debug
     options["syslog"] = arguments.syslog
@@ -94,6 +100,8 @@ def parse_options(argv):
                             else None
     options["conf_dir"] = os.path.abspath(arguments.conf_dir[0]) if arguments.conf_dir[0] else None
     options["no_color"] = arguments.no_color
+    options["check_mode"] = arguments.check_mode
+    options["dry_mode"] = arguments.dry_mode
 
     return options
 
@@ -115,16 +123,10 @@ def main():
 
     if options["subcommand"] in ("run", "verify"):
         workdir = misc.create_workdir(start_ts, configuration.conf, logger.logger)
-        Loggers.set_logging_to_file(logger, workdir, start_ts, configuration.conf)
-        if options["syslog"]:
-            # handler[1] - MemoryHandler, handler[3] - FileHandler
-            logger.logger.handlers[1].setTarget(logger.logger.handlers[3])
-        else:
-            # handler[0] - MemoryHandler, handler[2] - FileHandler
-            logger.logger.handlers[0].setTarget(logger.logger.handlers[2])
-        logger.logger.handlers[0].flush()
+        log_path = Loggers.set_logging_to_file(logger, workdir, start_ts, configuration.conf)
+        logger.flush_memory_handler(True, options["syslog"])
     else:
-        logger.logger.handlers[0].close()
+        logger.flush_memory_handler(False, options["syslog"])
 
     validators = Validators(logger.logger)
     validators.validate_options(options)
@@ -135,8 +137,9 @@ def main():
 
     user_groups = misc.get_all_user_groups(logger.logger)
 
-    if options["dry"]:
-        logger.logger.info("Skipping execution because of --dry-run option")
+    if options["conf_val"]:
+        logger.logger.info("Validation of configuration files was successful, program will exit now"
+                           ".")
         sys.exit(0)
 
     if options["subcommand"] == "show":
@@ -151,14 +154,14 @@ def main():
             if not validators.verify_task_permissions(selected_items, user_groups, config):
                 logger.logger.critical("Task forbidden")
                 sys.exit(errno.EPERM)
-            runner = Runners(logger.logger, lock, workdir, start_ts_raw,
-                             config["tasks"]["setup_hooks"])
-            if not options["self_setup"]:
-                runner.setup_ansible(selected_items["commit"])
-            lock.lock_inventory(lockpath)
             db_connector, db_path = DbSetup.connect_to_db(DbSetup(logger.logger, start_ts,
                                                           configuration.conf, options))
             db_writer = DbWriter(logger.logger, db_connector, db_path)
+            runner = Runners(logger.logger, lock, workdir, start_ts_raw,
+                             config["tasks"]["setup_hooks"], log_path, db_path)
+            if not options["self_setup"]:
+                runner.setup_ansible(selected_items["commit"], configuration.conf_dir)
+            lock.lock_inventory(lockpath)
             sequence_record_dict = runner.run_playitem(config, options, inv_file, lockpath,
                                                        db_writer)
             lock.unlock_inventory(lockpath)
