@@ -3,18 +3,21 @@
 import os
 import sys
 import subprocess
+from ansible_deployer.modules.globalvars import ANSIBLE_DEFAULT_CALLBACK_PLUGIN_PATH
 from ansible_deployer.modules.outputs.formatting import Formatters
 
 class Runners:
     """Class handling ansible hooks and ansible plays execution"""
 
-    def __init__(self, logger, lock_obj, workdir, start_ts_raw, setup_hooks):
+    def __init__(self, logger, lock_obj, workdir, start_ts_raw, setup_hooks, log_path, db_path):
         self.logger = logger
         self.lock_obj = lock_obj
         self.workdir = workdir
         self.start_ts_raw = start_ts_raw
         self.setup_hooks = setup_hooks
         self.sequence_id = os.path.basename(self.workdir)
+        self.log_path = log_path
+        self.db_path = db_path
 
     @staticmethod
     def reassign_commit_and_workdir(commit: str, workdir: str):
@@ -27,7 +30,7 @@ class Runners:
 
         return workdir, commit
 
-    def setup_ansible(self, commit: str):
+    def setup_ansible(self, commit: str, conf_dir: str):
         """
         Function responsible for execution of setup_hooks
         It passes the "commit" to the hook if one given, if not the hook should
@@ -39,10 +42,9 @@ class Runners:
         for hook in self.setup_hooks:
             if hook["module"] == "script":
                 try:
-                    with subprocess.Popen([hook["opts"]["file"], commit],
-                                          stdout=subprocess.PIPE,
-                                          stderr=subprocess.PIPE,
-                                          stdin=subprocess.PIPE) as proc:
+                    with subprocess.Popen([os.path.join(conf_dir, hook["opts"]["file"]), commit],
+                                          stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                                          stdin=subprocess.PIPE, cwd=workdir) as proc:
                         std_out, std_err = proc.communicate()
                         if proc.returncode != 0:
                             failed = True
@@ -116,6 +118,7 @@ class Runners:
         """
         playitems = self.get_playitems(config, options)
         host_list = []
+        returned = []
         if not playitems:
             self.logger.critical("No playitems found for requested task %s. Nothing to do.",
                                  options['task'])
@@ -129,46 +132,35 @@ class Runners:
                 command, command_env = self.construct_command(playitem, inventory, config, options)
                 self.logger.debug("Running '%s'.", command)
                 try:
-                    with subprocess.Popen(command, stdout=subprocess.PIPE,
-                                          stderr=subprocess.PIPE, env=command_env) as proc:
-                        returned = proc.communicate()
+                    with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                                          env=command_env) as proc:
+
+                        for msg in proc.stdout:
+                            dec_msg = msg.split(b"\n")[0].decode("utf-8")
+                            returned.append(dec_msg)
+                            if options["raw_output"]:
+                                print(dec_msg)
+
+                        proc.communicate()
                         format_obj = Formatters(self.logger)
-                        output, warning, error = Formatters.format_ansible_output(returned)
-                        play_host_list = db_writer.write_records(db_writer.parse_yaml_output(error,
-                                                                 self.sequence_id))
-                        for host in play_host_list:
-                            host_list.append(host)
+                        parsed_std = format_obj.format_ansible_output(returned)
+                        host_list = db_writer.parse_yaml_output_for_hosts(parsed_std["complete"],
+                                                                self.sequence_id)
                         sequence_records = db_writer.start_sequence_dict(host_list,
                                                                          self.setup_hooks, options,
                                                                          self.start_ts_raw,
                                                                          self.sequence_id)
-                        if options["raw_output"]:
-                            if proc.returncode != 0:
-                                if options["debug"]:
-                                    format_obj.debug_std_out(returned[0])
-                                self.logger.error("'%s' failed due to:", command)
-                                format_obj.format_std_err(returned[1])
-                                self.lock_obj.unlock_inventory(lockpath)
-                                db_writer.finalize_db_write(sequence_records, False)
-                                self.logger.critical("Program will exit now.")
-                                sys.exit(71)
-                            else:
-                                if options["debug"]:
-                                    format_obj.debug_std_err(returned[1])
-                                format_obj.format_std_out(returned[0])
-                                self.logger.info("'%s' ran succesfully", command)
+
+                        if proc.returncode == 0:
+                            format_obj.positive_ansible_output(parsed_std["warning"],
+                                                               parsed_std["output"], command)
                         else:
-                            if options["debug"]:
-                                format_obj.debug_std_out(returned[0])
-                                format_obj.format_std_err(returned[1])
-                            if proc.returncode == 0:
-                                format_obj.positive_ansible_output(warning, output, command)
-                            else:
-                                format_obj.negative_ansible_output(warning, error, command)
-                                self.lock_obj.unlock_inventory(lockpath)
-                                db_writer.finalize_db_write(sequence_records, False)
-                                self.logger.critical("Program will exit now.")
-                                sys.exit(71)
+                            format_obj.negative_ansible_output(parsed_std["warning"],
+                                                               parsed_std["error"], command)
+                            self.lock_obj.unlock_inventory(lockpath)
+                            db_writer.finalize_db_write(sequence_records, False)
+                            self.logger.critical("Program will exit now.")
+                            sys.exit(71)
                 except Exception as exc:
                     self.logger.critical("\"%s\" failed due to:")
                     self.logger.critical(exc)
@@ -183,22 +175,28 @@ class Runners:
     def get_tags_for_task(config: dict, options: dict):
         """Function to get task's tags"""
         tags = []
+        skip_tags = []
         for task in config["tasks"]["tasks"]:
             if task["name"] == options["task"]:
                 tags = task.get("tags", [])
 
-        return tags
+        if options["dry_mode"]:
+            tags.append("ansible_deployer_dry_mode")
 
-    @staticmethod
-    def construct_command(playitem: str, inventory: str, config: dict, options: dict):
+        for task in config["tasks"]["tasks"]:
+            if task["name"] == options["task"]:
+                skip_tags = task.get("skip_tags", [])
+
+        return tags, skip_tags
+
+    def construct_command(self, playitem: str, inventory: str, config: dict, options: dict):
         """Create final ansible command from available variables"""
-        tags = Runners.get_tags_for_task(config, options)
+        tags, skip_tags = Runners.get_tags_for_task(config, options)
 
         if "runner" in playitem and playitem["runner"] == "py.test":
             command = ["py.test", "--ansible-inventory", inventory]
             if options["limit"]:
-                command.append("--hosts='ansible://")
-                command.append(options["limit"]+"'")
+                command.append(f"--hosts='ansible://{options['limit']}'")
             else:
                 command.append("--hosts=ansible://all")
             command.append("--junit-xml=junit_"+options['task']+'.xml')
@@ -212,7 +210,21 @@ class Runners:
             if tags:
                 command.append("-t")
                 command.append(",".join(tags))
+            if skip_tags:
+                command.append("--skip-tags")
+                command.append(",".join(skip_tags))
+            if options["check_mode"]:
+                command.append("-C")
             command_env=dict(os.environ, ANSIBLE_STDOUT_CALLBACK="yaml", ANSIBLE_NOCOWS="1",
-                             ANSIBLE_LOAD_CALLBACK_PLUGINS="1")
+                             ANSIBLE_LOAD_CALLBACK_PLUGINS="1", LOG_PLAYS_PATH=self.log_path,
+                             ANSIBLE_CALLBACKS_ENABLED="log_plays_adjusted,sqlite_deployer",
+                             ANSIBLE_CALLBACK_PLUGINS=self.append_to_ansible_callbacks_path(),
+                             SQLITE_PATH=self.db_path, SEQUENCE_ID=self.sequence_id)
 
         return command, command_env
+
+    @staticmethod
+    def append_to_ansible_callbacks_path():
+        """Create final searchable path for ansible callback plugins"""
+        plugin_path = os.path.join(os.path.realpath(__file__).rsplit(os.sep, 3)[0], "plugins")
+        return f'{ANSIBLE_DEFAULT_CALLBACK_PLUGIN_PATH}:{plugin_path}'
